function varargout = MagResLearningTool(varargin)
% Copyright (C) 2013  Peter Casapicola, Bernhard Geiger
%                     Graz University of Technology
% Version 1.0
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.

gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @MagResLearningTool_OpeningFcn, ...
                   'gui_OutputFcn',  @MagResLearningTool_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%--------------------------------------------------------------------------
% --- Executes just before MagResLearningTool is made visible.
function MagResLearningTool_OpeningFcn(hObject, ~, handles, varargin)
%clc;
clearvars -global;
global state difficulty xyVec filterGiven 
global justStarted evaluated
%clear state difficulty xyVec filterGiven justStarted evaluated
state = 0; difficulty = 1;
justStarted = 1; evaluated = 0;
xyVec = [];
filterGiven.b = []; filterGiven.a = [];

handles.output = hObject;
storagePath = getStoragePath;
if exist(strcat(storagePath, '/results.csv'), 'file')
    A=importdata(strcat(storagePath, '/results.csv'),',');
    initcount = length(A);
else
    initcount = 0;
end
set(handles.round, 'String', initcount)

set(handles.bet, 'XLim', [-2*pi 2*pi]);
set(handles.bet, 'XTick', -2*pi:pi/2:2*pi);
set(handles.bet, 'XTickLabel', {'-2pi','-3pi/2','-pi','-pi/2','0',...
                                'pi/2','pi','3pi/2','2pi'})
                            

                            
set(gcf, 'KeyPressFcn', @onKeyDown)
set(gcf, 'WindowButtonMotionFcn', {@onMouseMove, handles});

enableButton(handles.evaluateButton, false);
enableButton(handles.filtered_sound, false);

handles.logo ;            % Auswahl des entsprechenden Axes-Objekts
G1=imread('logo.png','png');   % Einlesen der Grafik
 image(G1); 
 axis image;           % Grafik ausgeben, Grafik entzerre
 axis off;


 
% Update handles structure
guidata(hObject, handles);
movegui('center');
% user data management
uid = getUserID();
set(handles.idtext, 'String', uid);


%--------------------------------------------------------------------------
function onMouseMove(~, ~, handles)
currentCursor = get(gcf, 'Pointer');
if hittest(gcf) == handles.bet  % cursor is inside bet axes
    if ~strcmp(currentCursor, 'fullcrosshair')
        set(gcf, 'Pointer', 'fullcrosshair');
    end
    pt = get(handles.bet, 'CurrentPoint');
    set(handles.betX, 'String', sprintf('%1.2f',pt(1,1)));
    set(handles.betY, 'String', sprintf('%1.2f',pt(1,2)));
else
    if ~strcmp(currentCursor, 'arrow')
        set(gcf, 'Pointer', 'arrow');
        set(handles.betX, 'String', '-');
        set(handles.betY, 'String', '-');
    end
end

%--------------------------------------------------------------------------
function onKeyDown(~, ~, handles)
key = double(get(gcf, 'CurrentCharacter'));
if ~isempty(key)
    switch (key)
        case 13 % Enter
            evaluate(handles);
    end
end

%--------------------------------------------------------------------------
% --- Outputs from this function are returned to the command line.
function varargout = MagResLearningTool_OutputFcn(~, ~, handles) 
% Get default command line output from handles structure
varargout{1} = handles.output;

% -------------------------------------------------------------------------
function exit_Callback(~, ~, ~)
selection = questdlg('Do you want to close the GUI?',...
                     'Close Request Function','Yes','No','Yes');
switch selection
    case 'Yes'
        delete(gcf);
        fclose('all');
    case 'No'
        return;
end
        
%--------------------------------------------------------------------------
% --- Executes on button press in startButton.
function startButton_Callback(~, ~, handles)
global state difficulty xyVec filterGiven evaluated justStarted
global points evaluation
points = 0;
state = 1;
xyVec = [];
resetDisplay(handles, false); % reset statistics display
set(handles.evaluateButton, 'Enable', 'off');
enableDifficultyRadioButtons(handles, false);
round_cnt = str2double(get(handles.round, 'String'));

if evaluated || justStarted
    round_cnt = round_cnt+1;
    set(handles.round, 'String', round_cnt);
    evaluated = 0; justStarted = 0;
end
    
cla(handles.zp);

evaluation.type = ceil(8*rand);
[b, a] = makeFilter(difficulty, evaluation.type);
filterGiven.b = b; filterGiven.a = a;

zplane(b, a, handles.zp);

a_max = abs(roots(a)); b_max = abs(roots(b));
axis_size = 1.2;
if ~isempty(a_max)
    axis_size = max(max(a_max+0.2), axis_size);
end
if ~isempty(b_max)
      axis_size = max(max(b_max)+0.2, axis_size);
end  
axis(handles.zp, [-axis_size axis_size -axis_size axis_size]);

w = -2*pi:0.05:2*pi;
cla(handles.bet);
axes(handles.bet);
hold on;

plot(handles.bet, w, ones(1, length(w)), 'k--');
plot(handles.bet, w, zeros(1, length(w)), 'k--');
line([-pi -pi], [-0.1 1.1], 'Color', [.8 .8 .8])
line([pi pi], [-0.1 1.1], 'Color', [.8 .8 .8])
axis(handles.bet, [min(w) max(w) -0.1 1.1])
set(handles.bet, 'YLim', [-0.1 1.1]);
set(get(handles.bet,'children'),'HitTest','off');
enableButton(handles.filtered_sound, true);

%--------------------------------------------------------------------------
% --- Executes on button press in evaluateButton.
function evaluateButton_Callback(~, ~, handles)
enableButton(handles.evaluateButton, 0);
evaluate(handles);
enableDifficultyRadioButtons(handles, true);



%--------------------------------------------------------------------------
% --- Executes when selected object is changed in diffSelector.
function diffSelector_SelectionChangeFcn(~, eventdata, ~)
global difficulty
tag = get(eventdata.NewValue, 'Tag');
switch tag % Get Tag of selected object.
    case 'rookie'
        difficulty = 1;
    case 'intermediate'
        difficulty = 2;
    case 'advanced'
        difficulty = 3;
    case 'expert'
        difficulty = 4;
end

%--------------------------------------------------------------------------
% --- Executes on mouse press over figure background, over a disabled or
% --- inactive control, or over an axes background.
% function figure1_WindowButtonDownFcn(~, ~, handles)
% global state difficulty xyVec filterUser points
% if state ~= 1 || hittest(gcf) ~= handles.bet
%     return;
% end
% 
% [x, y] = getBetMousePosition(handles);
% w = -2*pi:0.05:2*pi;
% switch get(gcf, 'SelectionType')
%     case 'normal'   % left mouse click, add current point
%         xyVec = [xyVec; x y];
%         if difficulty <= 4 % Automatically ensure 2pi-periodicity
%             xyVec = [xyVec; x+2*pi y; x-2*pi y];
%             if difficulty <= 2 % Automatically ensure 2pi-periodicity and even symmetry
%                 xyVec = [xyVec; -x y; -x+2*pi y; -x-2*pi y];
%             end
%         end
%         points = points+1;
%     case 'alt'      % right mouse click, delete last point
%         if length(xyVec(:,1)) >= 1
%             switch difficulty 
%                 case {1, 2}
%                     xyVec = xyVec(1:end-6, :);
%                 case {3, 4}
%                     xyVec = xyVec(1:end-3, :);
% %                 case 4
% %                     xyVec = xyVec(1:end-1, :);
%             end
%             points = points-1;
%         end
% end
% 
% % update evaluation button enabled state
% enableButton(handles.evaluateButton, ~isempty(xyVec));
% 
% cla(handles.bet);
% 
% plot(w, 1, 'k--')
% hold on;
% % grid on;
% plot(w, 0, 'k--')
% plot(xyVec(:,1), xyVec(:,2), 'rx')
% line([-pi -pi], [-0.1 1.1], 'Color', [.8 .8 .8])
% line([pi pi], [-0.1 1.1], 'Color', [.8 .8 .8])
% axis([min(w) max(w) -0.1 1.1])
% if length(xyVec(:,1)) > 1
%     h_hat = interp1(xyVec(:,1), xyVec(:,2), w, 'pchip');
%     h_hat(h_hat < 0) = 0;
%     h_hat(h_hat > 1) = 1;
%     plot(w, h_hat, 'r');
%     filterUser.h_hat = h_hat;
% end
% 
% set(handles.int_points, 'String', points)
% % disable HitTest for objects inside bet axes
% set(get(handles.bet,'children'), 'HitTest', 'off');

%--------------------------------------------------------------------------
% get mouse position relative to bet axes
function [x,y] = getBetMousePosition(handles)
mousePos = get(handles.bet, 'CurrentPoint');
x = mousePos(1,1);
y = mousePos(1,2);

%--------------------------------------------------------------------------
% enable or disable button
function enableButton(handle, value)
if value == false
    set(handle, 'Enable', 'off');
elseif value == true
    set(handle, 'Enable', 'on');
end

%--------------------------------------------------------------------------
% enable or disable button
function enableDifficultyRadioButtons(handles, value)
children = get(handles.diffSelector, 'Children');
if value == false
    set(children, 'Enable', 'off');
elseif value == true
    set(children, 'Enable', 'on');
end

%--------------------------------------------------------------------------
% reset displays
function resetDisplay(handles, full)
set(handles.error, 'String', '-');
set(handles.int_points, 'String', '0');
if full == true
    set(handles.avg_err, 'String', '-');
end
set(handles.gmax, 'String', '-','ForegroundColor','black');
set(handles.gmin, 'String', '-','ForegroundColor','black');
set(handles.nrlmax, 'String', '-/-');
set(handles.nrlmin, 'String', '-/-');

%--------------------------------------------------------------------------
% evaluate user designed filter
function evaluate(handles)
global filterGiven filterUser state evaluated evaluation
b = filterGiven.b; a = filterGiven.a;
h_hat = filterUser.h_hat;

% update round count
round_cnt = str2double(get(handles.round, 'String'));

w = -2*pi:0.05:2*pi;
h = freqz(b, a, w);
h = h./max(abs(h));
plot(w, abs(h), 'b');

minpiind = find(abs(w+pi)<0.1, 1, 'first'); % find first pi index
maxpiind = find(abs(w-pi)<0.1, 1, 'last');  % find last pi index

plot(w, h_hat, 'r');

temp = abs(abs(h)-h_hat);
h_diff = temp(minpiind:maxpiind);
Sigma_y = 100/(2*pi) * trapz(w(minpiind:maxpiind),h_diff.^2);
evaluation.error = Sigma_y;

set(handles.error, 'String', sprintf('%3.3f', Sigma_y));  % output estimation error
avg_err = str2double(get(handles.avg_err,'String'));
if strcmp(get(handles.avg_err, 'String'), '-') == true
    avg_err = 0;
end
avg_err = (round_cnt-1)/round_cnt*avg_err + Sigma_y/round_cnt;
set(handles.avg_err, 'String', sprintf('%3.3f',avg_err));    % output mean estimation error

[globMax, globMin, lmax, lmin] = findExtremes(abs(h(minpiind:maxpiind)));
[globMaxH, globMinH, lmaxH, lminH] = findExtremes(h_hat(minpiind:maxpiind));
plot(w(globMax+minpiind-1), abs(h(globMax+minpiind-1)),'bo');
plot(w(globMin+minpiind-1), abs(h(globMin+minpiind-1)),'bo');
plot(w(globMaxH+minpiind-1), (h_hat(globMaxH+minpiind-1)),'bo');
plot(w(globMinH+minpiind-1), (h_hat(globMinH+minpiind-1)),'bo');
plot(w(lmax+minpiind-1), abs(h(lmax+minpiind-1)),'g*');
plot(w(lmin+minpiind-1), abs(h(lmin+minpiind-1)),'g*');

if abs(globMax-globMaxH) < 10
    set(handles.gmax,'String','OK','ForegroundColor','green');
else
    set(handles.gmax,'String','Fail!','ForegroundColor','red');
end

if abs(globMin-globMinH) < 10
    set(handles.gmin,'String','OK','ForegroundColor','green');
else
    set(handles.gmin,'String','Fail!','ForegroundColor','red');
end

set(handles.nrlmax,'String', sprintf('%d/%d',length(lmax),length(lmaxH)));
set(handles.nrlmin,'String', sprintf('%d/%d',length(lmin),length(lminH)));

evaluation.gmax = get(handles.gmax, 'String');
evaluation.gmin = get(handles.gmin, 'String');
evaluation.nrlmax = get(handles.nrlmax, 'String');
evaluation.nrlmin = get(handles.nrlmin, 'String');

state = 0; evaluated = 1;
% disable HitTest for objects inside bet axes
set(get(handles.bet, 'children'), 'HitTest', 'off');

storeResults; % store results locally

if get(handles.feedbackCheckbox,'Value') == 1.0
    sendFeedback;	% send results to statistics server
end

%--------------------------------------------------------------------------
% --- Executes when user attempts to close figure1.
function figure1_CloseRequestFcn(hObject, ~, ~)
fclose('all');      % close open files
delete(hObject);

%--------------------------------------------------------------------------
% send feedback to statistics server
function sendFeedback
global userID evaluation difficulty points;
% wrap evaluation data
checksum = lower(dec2hex(bitand(sum(userID(:)-0), 255)));
data = {'userID', strcat(userID, checksum),...
        'difficulty', num2str(difficulty),...
        'type', num2str(evaluation.type),...
        'points', num2str(points),...
        'error', num2str(evaluation.error),...
        'gmax', evaluation.gmax,...
        'gmin', evaluation.gmin,...
        'nrlmax', evaluation.nrlmax,...
        'nrlmin', evaluation.nrlmin};

try
	url = 'http://www2.spsc.tugraz.at/people/spacetec/statistics.php';
    disp(urlread(url, 'post', data));
catch err
    disp('Warning: could not connect to statistics server'); %err);
end

%--------------------------------------------------------------------------
% store current result in csv-file
function storeResults
global evaluation difficulty points;
storagePath = getStoragePath;
if ~exist(strcat(storagePath, '/results.csv'), 'file')
    storageFile = fopen(strcat(storagePath, '/results.csv'), 'a');
    fprintf(storageFile,...
        'Datum,Schwierigkeitsstufe,Filtertyp,Interpolationspunkte,Sch�tzfehler,Globales Maximum,Globales Minimum,Anzahl Maxima,Anzahl Minima\n');
else
    storageFile = fopen(strcat(storagePath, '/results.csv'), 'a');
end
try
    fprintf(storageFile, '%s,%i,%i,%i,%3.3f,%s,%s,%s,%s\n',...
		datestr(now), difficulty, evaluation.type, points, evaluation.error,...
		evaluation.gmax, evaluation.gmin, evaluation.nrlmax,...
		evaluation.nrlmin);
    fclose(storageFile);
catch err
    disp(err);
end

%--------------------------------------------------------------------------
% get user directory
function dir = userDir
if ispc
    dir = winqueryreg('HKEY_CURRENT_USER',...
        ['Software\Microsoft\Windows\CurrentVersion\' ...
         'Explorer\Shell Folders'], 'Personal');
else
    dir = char(java.lang.System.getProperty('user.home'));
end

%--------------------------------------------------------------------------
% return local storage path (and create it in advance if not found)
function path = getStoragePath
path = strcat(userDir, '/MagResLTUserData');
if ~exist(path, 'dir')
    mkdir(path);
end

%--------------------------------------------------------------------------
% user data management
function uid = getUserID
global userID
storagePath = getStoragePath();
if ~exist(strcat(storagePath, '/userID.dat'), 'file')
    userID = generateUserID();
    uid = userID;
    storageFile = fopen(strcat(storagePath, '/userID.dat'), 'w');
    fileattrib(strcat(storagePath, '/userID.dat'));
    fprintf(storageFile, '%s', userID);
    fclose(storageFile);
else
    storageFile = fopen(strcat(storagePath, '/userID.dat'), 'r');
    userID = fscanf(storageFile, '%s');
    uid = userID;
    fclose(storageFile);
    if ~isUserIDValid(userID)   % if user id is not valid generate new one
        userID = generateUserID();
        uid =UserID;
        storageFile = fopen(strcat(storagePath, '/userID.dat'), 'w');
        fprintf(storageFile, '%s',  userID);
        fclose(storageFile); 
    end
end

%--------------------------------------------------------------------------
% generate a random user id
function id = generateUserID
id = char(java.util.UUID.randomUUID);

%--------------------------------------------------------------------------
% check if id is valid
function valid = isUserIDValid(id)
valid = ~isempty(regexpi(id,'^[A-Fa-f0-9-]{36}$', 'match'));

%--------------------------------------------------------------------------
% open the manual PDF
function help_Callback(~, ~, ~)
open('MagResLearningTool.pdf');


% --- Executes on button press in soundButton.
function soundButton_Callback(~, ~, ~)
S = load('gong.mat');
sound(S.y);




% --- Executes on button press in statsButton.
function statsButton_Callback(~, ~, ~)
% plotfghObject    handle to statsButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

    storagePath = getStoragePath;
    A=importdata(strcat(storagePath, '/results.csv'),',');

    Schtzfehler=zeros(1,size(A,1));
    for ind=1:size(A,1)
        Amat(ind,:)=strsplit(A{ind},',');
        Schtzfehler(ind)=str2double(Amat(ind,5));
        Schwierigkeitsstufe(ind)=str2double(Amat(ind,2));
    end


    figure(2)
    % title('Error of Estimation for User:"d091bb44e7" ');
    % subplot(2,4,1:4); 
    % ylabel('Error of Estimation')
    % xlabel('Number of Tries')
    Schtzfehler = Schtzfehler(Schtzfehler>0);
    Schwierigkeitsstufe = Schwierigkeitsstufe(Schtzfehler>0);

    for k=1:4
        subplot(2,4,k);
        hold all;
        if ~isempty(Schtzfehler(Schwierigkeitsstufe == k))
            bar(Schtzfehler(Schwierigkeitsstufe == k),'w');
            axis([max(0,length(Schtzfehler(Schwierigkeitsstufe == k))-50) length(Schtzfehler(Schwierigkeitsstufe == k)) 0 30])
            hold on;
            plot(filter(1/10*ones(1,10),1,Schtzfehler(Schwierigkeitsstufe == k)),'r','LineWidth',2)
        else
            bar(0,'w');
            axis([0 1 0 30]);
        end
        title(sprintf('Difficulty %d',k),'FontSize',14)
        if k==1
            ylabel('Estimation Error','FontSize',12)
            xlabel('Rounds','FontSize',12)
        end
        
    end;
    EstMatrix=nan(max([length(Schtzfehler(Schwierigkeitsstufe == 1)),length(Schtzfehler(Schwierigkeitsstufe == 2)),length(Schtzfehler(Schwierigkeitsstufe == 3)),length(Schtzfehler(Schwierigkeitsstufe == 4))]),4);
    EstMatrix(1:length(Schtzfehler(Schwierigkeitsstufe == 1)),1)=Schtzfehler(Schwierigkeitsstufe == 1);
    EstMatrix(1:length(Schtzfehler(Schwierigkeitsstufe == 2)),2)=Schtzfehler(Schwierigkeitsstufe == 2);
    EstMatrix(1:length(Schtzfehler(Schwierigkeitsstufe == 3)),3)=Schtzfehler(Schwierigkeitsstufe == 3);
    EstMatrix(1:length(Schtzfehler(Schwierigkeitsstufe == 4)),4)=Schtzfehler(Schwierigkeitsstufe == 4);
    subplot(2,4,5:8);
    boxplot(EstMatrix,'datalim',[0,30],'extrememode','compress');
    xlabel('Difficulty','FontSize',12)
    ylabel('Estimation Error','FontSize',12)


% --- Executes on button press in filtered_sound.
function filtered_sound_Callback(hObject, eventdata, handles)
global filterGiven
S = load('gong.mat');
b = filterGiven.b;
a = filterGiven.a;
y1 = filter(b,a,S.y);
sound(y1);


% --- Executes on mouse press over axes background.
function bet_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to bet (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global state difficulty xyVec filterUser points
if state ~= 1 || hittest(gcf) ~= handles.bet
    return;
end

[x, y] = getBetMousePosition(handles);
w = -2*pi:0.05:2*pi;
switch get(gcf, 'SelectionType')
    case 'normal'   % left mouse click, add current point
        xyVec = [xyVec; x y];
        if difficulty <= 4 % Automatically ensure 2pi-periodicity
            xyVec = [xyVec; x+2*pi y; x-2*pi y];
            if difficulty <= 2 % Automatically ensure 2pi-periodicity and even symmetry
                xyVec = [xyVec; -x y; -x+2*pi y; -x-2*pi y];
            end
        end
        points = points+1;
    case 'alt'      % right mouse click, delete last point
        if length(xyVec(:,1)) >= 1
            switch difficulty 
                case {1, 2}
                    xyVec = xyVec(1:end-6, :);
                case {3, 4}
                    xyVec = xyVec(1:end-3, :);
%                 case 4
%                     xyVec = xyVec(1:end-1, :);
            end
            points = points-1;
        end
end

% update evaluation button enabled state
enableButton(handles.evaluateButton, ~isempty(xyVec));

cla(handles.bet);

plot(w, 1, 'k--')
hold on;
% grid on;
plot(w, 0, 'k--')
plot(xyVec(:,1), xyVec(:,2), 'rx')
line([-pi -pi], [-0.1 1.1], 'Color', [.8 .8 .8])
line([pi pi], [-0.1 1.1], 'Color', [.8 .8 .8])
axis([min(w) max(w) -0.1 1.1])
if length(xyVec(:,1)) > 1
    h_hat = interp1(xyVec(:,1), xyVec(:,2), w, 'pchip');
    h_hat(h_hat < 0) = 0;
    h_hat(h_hat > 1) = 1;
    plot(w, h_hat, 'r');
    filterUser.h_hat = h_hat;
end

set(handles.int_points, 'String', points)
% disable HitTest for objects inside bet axes
set(get(handles.bet,'children'), 'HitTest', 'off');


% --- Executes on key press with focus on startButton and none of its controls.
function startButton_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to startButton (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)
